from matmul import matmul_partitioned, matmul
from matmul_pure import matmul as matmul_pure

import numpy as np
from numpy.testing import assert_array_almost_equal

from timeit import timeit

import threading

A = np.random.random((800, 200))
B = np.random.random((200, 300))
C = np.zeros((800, 300))
D = np.zeros_like(C)

matmul_partitioned(0, len(A), A, B, C)

# First, test accuracy
assert_array_almost_equal(np.dot(A, B), C)

matmul(A, B, D)
assert_array_almost_equal(np.dot(A, B), D)

print "Accuracy: all good!"

# Now, test timing
def t0():
    matmul_pure(A, B, C)

def t1():
    matmul_partitioned(0, len(A), A, B, C)

def t2():
    N = len(A)

    a = threading.Thread(target=matmul_partitioned,
                         args=(0, N//2, A, B, C))
    b = threading.Thread(target=matmul_partitioned,
                         args=(N//2, N, A, B, C))

    a.start()
    b.start()
    a.join()
    b.join()

    assert (not a.is_alive()) and (not b.is_alive())


def t3():
    np.dot(A, B)

N = 5
#time0 = timeit(stmt="t0()", setup="from __main__ import t0", number=N)
time1 = timeit(stmt="t1()", setup="from __main__ import t1", number=N)
time2 = timeit(stmt="t2()", setup="from __main__ import t2", number=N)
time3 = timeit(stmt="t3()", setup="from __main__ import t3", number=N)

print "Parallel accuracy test:",
C.fill(0)
t2()
assert_array_almost_equal(C, np.dot(A, B))
print "OK"

#print "Pure Python matmul:", time0
print "Cython matmul, no parallel:", time1
print "Cython matmul, 2 threads:", time2
print "But ATLAS still kicks butt:", time3

